package org.imixs.workflow.jee.rest.jaxrs;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.Vector;

import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.xml.EntityTable;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * This MessageBodyWriter generates an HTML representation from a EntityTable.
 * The output is a table format where each entity has the same columns.
 * 
 * @author rsoika
 * 
 */
@Provider
@Produces("text/html")
public class EntityTableWriter implements MessageBodyWriter<EntityTable> {

	public boolean isWriteable(Class<?> type, Type genericType,
			Annotation[] annotations, MediaType mediaType) {
		return EntityTable.class.isAssignableFrom(type);
	}

	public void writeTo(EntityTable entityCollection, Class<?> type,
			Type genericType, Annotation[] annotations, MediaType mediaType,
			MultivaluedMap<String, Object> httpHeaders,
			OutputStream entityStream) throws IOException,
			WebApplicationException {

		boolean trClass = true;

		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(
				entityStream));

		bw.write("<html>");
		XMLItemCollectionWriter.printHead(bw);

		bw.write("<body>");

		/* Print table header */
		bw.write("<table><tbody>");
		if (trClass)
			bw.write("<tr class=\"a\">");
		else
			bw.write("<tr class=\"b\">");
		trClass = !trClass;

		for (String attr : entityCollection.getAttributeList()) {
			bw.write("<th>" + attr + "</th>");
		}
		bw.write("</tr>");

		// print table body
		try {

			for (XMLItemCollection xmlworkItem : entityCollection.getEntity()) {
				/* Print row */
				if (trClass)
					bw.write("<tr class=\"a\">");
				else
					bw.write("<tr class=\"b\">");
				trClass = !trClass;

				for (String attr : entityCollection.getAttributeList()) {
					ItemCollection itemCol = XMLItemCollectionAdapter
							.getItemCollection(xmlworkItem);
					Vector vValues = itemCol.getItemValue(attr);
					/*
					 * String sValue = vValues.toString(); sValue =
					 * sValue.substring(1, sValue.length() - 1);
					 */
					bw.write("<td>"
							+ XMLItemCollectionWriter
									.convertValuesToString(vValues) + "</td>");
				}
				bw.write("</tr>");

			}

			bw.write("</tbody></table>");
		} catch (Exception e) {
			bw.write("ERROR<br>");
			// e.printStackTrace(bw.);
		}

		bw.write("</body>");
		bw.write("</html>");

		bw.flush();
	}

	public long getSize(EntityTable arg0, Class<?> arg1, Type arg2,
			Annotation[] arg3, MediaType arg4) {
		return -1;
	}

}
