package org.imixs.workflow.jee.rest.servlet;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

/**
 * This Abstract Handler Class should be subclassed by the Model, Report and
 * Workflow Handler. The Abstract RestSerivce Handler supports general helper
 * methods used by all tree modules.
 * 
 * @version 0.0.1
 * @author rsoika
 * 
 */
public abstract class AbstractRestServiceHandler {
	URIHandler uriHandler=null;
	

	public AbstractRestServiceHandler(URIHandler aurihandler) {
		super();
		
		uriHandler=aurihandler;
	}

	/**
	 * This Method prints a single wokrItem in html format
	 * 
	 * @param out
	 * @param workItem
	 */
	protected void printWorkItemHTML(PrintWriter out, ItemCollection workItem) {
		boolean trClass = false;

		out.println("<table><tbody>");

		out.println("<tr class=\"a\">");

		out.println("<th colspan=\"2\"><b>Workitem UniqueID="
				+ workItem.getItemValueString("$uniqueid") + "</b></th></tr>");

		out.println("<tr class=\"a\">");

		out.println("<th>Property</th><th>Value</th></tr>");

		Iterator iter = workItem.getAllItems().entrySet().iterator();
		while (iter.hasNext()) {

			// WorkItemAttribute da = new WorkItemAttribute();
			Map.Entry mapEntry = (Map.Entry) iter.next();
			String sName = mapEntry.getKey().toString();
			Vector value = (Vector) mapEntry.getValue();

			if (trClass)
				out.println("<tr class=\"a\">");
			else
				out.println("<tr class=\"b\">");
			trClass = !trClass;

			out.println("<td>" + sName + "</td><td>"
					+ convertValuesToString(value) + "</td></tr>");

		}
		out.println("</tbody></table>");

		out.println("<br/><br/>");
	}
	
	/**
	 * This Method prints a single wokrItem in xml format. The method uses the
	 * DataObjectUtil Class from the org.imixs.workflow.services.client_1.0
	 * module
	 * 
	 * @param out
	 * @param workItem
	 * @throws Exception
	 */
	protected void printWorkItemXML(ItemCollection workItem) throws Exception {

		String sXMLResult = getWorkItemXML(workItem);

		// Transform xml ?
		if (uriHandler.getXsl() != null
				&& !"".equals(uriHandler.getXsl())) {
			// reset contentype now - can be specified by query param
			// 'contenttype'
			String sContentType = uriHandler.getContenttype();
			if (sContentType == null || "".equals(sContentType))
				sContentType = "text/html";

			uriHandler.getResponse().setContentType(sContentType);

			uriHandler.getResponse().setHeader("Content-Type", sContentType + ";charset="
					+ uriHandler.getEncoding());
			// settings to enable MS IE
			uriHandler.getResponse().setHeader("Pragma", "public");
			uriHandler.getResponse().setHeader("Cache-Control", "must-revalidate");

			// set Attachment filename header for direct download?
			String sDownloadName = uriHandler.getDownload();
			if (sDownloadName != null && !"".equals(sDownloadName))
				uriHandler.getResponse().setHeader("Content-disposition",
						"attachment;filename=" + sDownloadName);

			uriHandler.getResponse().setCharacterEncoding(uriHandler.getEncoding());
			this.xslTranformationFromFile(sXMLResult, uriHandler
					.getXsl(), uriHandler.getEncoding());
		} else {
			// print xml
			PrintWriter out = uriHandler.getResponse().getWriter();

			uriHandler.getResponse().setCharacterEncoding(uriHandler.getEncoding());
			uriHandler.getResponse().setContentLength(sXMLResult.length());
			uriHandler.getResponse().setContentType("text/xml");
			uriHandler.getResponse().setHeader("Content-Type", "text/xml;charset="
					+ uriHandler.getEncoding());

			// settings to enable MS IE
			uriHandler.getResponse().setHeader("Pragma", "public");
			uriHandler.getResponse().setHeader("Cache-Control", "must-revalidate");
			out.println(sXMLResult);
			out.close();
		}

	}

	/**
	 * This Method prints a single wokrItem in simple text format
	 * 
	 * @param out
	 * @param workItem
	 * @throws IOException
	 */
	protected void printWorkItemText(ItemCollection workItem) throws IOException {

		PrintWriter out = uriHandler.getResponse().getWriter();
		out.println("[" + workItem.getItemValueString("$uniqueid") + "]");
		Iterator iter = workItem.getAllItems().entrySet().iterator();
		while (iter.hasNext()) {
			Map.Entry mapEntry = (Map.Entry) iter.next();
			String sName = mapEntry.getKey().toString();
			Vector value = (Vector) mapEntry.getValue();
			out.println("" + sName + "= " + convertValuesToString(value) + "");
		}

		out.println("");
	}


	

	/**
	 * This Method prints a collection of workitems in html format
	 * 
	 * @param out
	 * @param workItem
	 * @throws IOException
	 */
	protected void printWorkItemCollectionHTML(Collection<ItemCollection> col)
			throws IOException {
		PrintWriter out = uriHandler.getResponse().getWriter();
		out.println("<html>");
		printHead(out);
		out.println("<body>");
		try {
			out.println("<h1>" + uriHandler.getSubresource() + "</h1>");
			out.println("<h2>" + col.size() + " Entries</h2>");

			for (ItemCollection workItem : col) {
				printWorkItemHTML(out, workItem);

			}
		} catch (Exception e) {
			out.println("ERROR<br>");
			e.printStackTrace(out);
		}
		out.println("<hr>");

		out.println("</body>");
		out.println("</html>");
		out.close();

	}

	/**
	 * This Method prints a collection of workitems in html table format the
	 * attributeList contains the fieldnames to be printed into the table. Each
	 * Attribute will be represented by seperate column.
	 * 
	 * @param out
	 * @param workItem
	 * @throws IOException
	 */
	protected void printWorkItemCollectionHTML(Collection<ItemCollection> col,
			Vector<String> attributList) throws IOException {

		boolean trClass = true;
		PrintWriter out = uriHandler.getResponse().getWriter();
		out.println("<html>");
		printHead(out);
		out.println("<body>");

		/* Print table header */
		out.println("<table>");
		if (trClass)
			out.println("<tr class=\"a\">");
		else
			out.println("<tr class=\"b\">");
		trClass = !trClass;

		for (String attr : attributList) {
			out.println("<th>" + attr + "</th>");
		}
		out.println("</tr>");

		try {

			for (ItemCollection workItem : col) {

				/* Print row */
				if (trClass)
					out.println("<tr class=\"a\">");
				else
					out.println("<tr class=\"b\">");
				trClass = !trClass;

				for (String attr : attributList) {
					Vector vValues = workItem.getItemValue(attr);
					// remove [ ]
					/*
					 * String sValue = vValues.toString(); sValue =
					 * sValue.substring(1, sValue.length() - 1);
					 */
					out.println("<td>" + convertValuesToString(vValues)
							+ "</td>");
				}
				out.println("</tr>");

			}
		} catch (Exception e) {
			out.println("ERROR<br>");
			e.printStackTrace(out);
		}

		out.println("</body>");
		out.println("</html>");
		out.close();

	}

	/**
	 * This Method prints a collection of workitems in xml format. The method
	 * uses the DataObjectUtil Class from the
	 * org.imixs.workflow.services.client_1.0 module
	 * 
	 * @param out
	 * @param workItem
	 * @throws Exception
	 */
	protected void printWorkItemCollectionXML(Collection<ItemCollection> col)
			throws Exception {
		PrintWriter out = uriHandler.getResponse().getWriter();
		out.println(getWorkItemCollectionXML(col));
		out.close();

	}

	/**
	 * This Method prints a collection of workitems in html format
	 * 
	 * @param out
	 * @param workItem
	 * @throws IOException
	 */
	protected void printWorkItemCollectionText(Collection<ItemCollection> col)
			throws IOException {
		for (ItemCollection workItem : col) {
			printWorkItemText(workItem);

		}
	}

	
	
	/**
	 * This method returns the XML String generated from a ItemCollection
	 * 
	 * @param workItem
	 * @return
	 * @throws Exception
	 */
	private String getWorkItemXML(ItemCollection workItem) throws Exception {
		
		XMLItemCollection xmlItemCollection= XMLItemCollectionAdapter.putItemCollection(workItem);
		
		StringWriter writer = new StringWriter();

		JAXBContext context = JAXBContext.newInstance(XMLItemCollection.class);
		Marshaller m=context.createMarshaller();
		m.marshal(xmlItemCollection,writer);
		
		return writer.toString();
	
	}

	/**
	 * This method returns the XML String generated from a collection of
	 * ItemCollections
	 * 
	 * @param workItem
	 * @return
	 * @throws Exception
	 */
	protected String getWorkItemCollectionXML(Collection<ItemCollection> col)
			throws Exception {
		
		
		EntityCollection entCol=buildEntityCollection(col);
		
		StringWriter writer = new StringWriter();

		JAXBContext context = JAXBContext.newInstance(EntityCollection.class);
		Marshaller m=context.createMarshaller();
		m.marshal(entCol,writer);
		
		return writer.toString();
		

	}
	
	/**
	 * This method converts the Values of a vector into a string representation.
	 * 
	 * Multivalues will be separated with '~' characters. Date Objects will be
	 * converted into a short String representation taking the server locale
	 * 
	 * 
	 * @param values
	 * @return
	 */
	private String convertValuesToString(Vector values) {
		String convertedValue = "";

		if (values == null)
			return convertedValue;

		boolean bFirstValue = true;
		// Iterate over vector list
		Iterator iter = values.iterator();
		while (iter.hasNext()) {
			Object o = iter.next();
			Date dateValue = null;
			// now test the objct type to date
			if (o instanceof Date) {
				dateValue = (Date) o;
			}
			if (o instanceof Calendar) {
				Calendar cal = (Calendar) o;
				dateValue = cal.getTime();
			}

			// if it is not the first value then add the delimiter ~
			String singleValue = "";
			if (!bFirstValue)
				convertedValue += "~";

			// if value is a date object format date into a string
			// otherwise take the value as it is
			if (dateValue != null) {
				singleValue = DateFormat.getDateTimeInstance(DateFormat.SHORT,
						DateFormat.SHORT).format(dateValue);
			} else {
				// take value as it is
				if (o != null)
					singleValue = singleValue + o.toString();
			}

			convertedValue += singleValue;
			bFirstValue = false;
		}

		// return values as string
		return convertedValue;
	}

	
	/**
	 * Helper method to convert a Collection of
	 * org.imixs.workflow.jee.ejb.Entity Objects into a EntityCollection
	 * 
	 * @param col
	 * @return
	 * @throws Exception
	 */
	private EntityCollection buildEntityCollection(
			Collection<ItemCollection> col) throws Exception {
		EntityCollection wl = new EntityCollection();
		Iterator it = col.iterator();
		int max = col.size();
		int i = 0;
		XMLItemCollection[] entities = new XMLItemCollection[max];
		while (it.hasNext()) {
			ItemCollection icw = (ItemCollection) it.next();
			XMLItemCollection entity = XMLItemCollectionAdapter.putItemCollection(icw);
			entities[i] = entity;
			i++;
		}
		if (max > 0)
			wl.setEntity(entities);
		return wl;
	}
	

	

	protected void setContentType() {
		if ("xml".equals(uriHandler.getFormat()))
			uriHandler.getResponse().setContentType("text/xml");
		else if ("html".equals(uriHandler.getFormat()))
			uriHandler.getResponse().setContentType("text/html");
		else
			uriHandler.getResponse().setContentType("text/text");
	}

	

	public static void printHead(PrintWriter out) {
		out.println("<head>");
		out.println("<style>");
		out
				.println("table {padding:0px;width: 100%;margin-left: -2px;margin-right: -2px;}");
		out
				.println("body,td,select,input,li {font-family: Verdana, Helvetica, Arial, sans-serif;font-size: 13px;}");
		out
				.println("table th {color: white;background-color: #bbb;text-align: left;font-weight: bold;}");

		out.println("table th,table td {font-size: 12px;}");

		out.println("table tr.a {background-color: #ddd;}");

		out.println("table tr.b {background-color: #eee;}");

		out.println("</style>");
		out.println("</head>");
	}
	
	
	/**
	 * This method transforms an xml string by a xsltSourceFile
	 * 
	 * The XMLSource will be read from a InputStreamReader object to provide the
	 * XSLTransformer with a inputsream in expected encoding.
	 * 
	 * @param xmlSource
	 * @param xsltSourceFile
	 * @return
	 */
	protected void xslTranformationFromFile( String xmlSource,
			String xsltSourceFile, String aEncoding) {
		try {
			TransformerFactory transFact = TransformerFactory.newInstance();
			// generate InputStream Reader with encoding
			ByteArrayInputStream bais = new ByteArrayInputStream(xmlSource
					.getBytes());
			InputStreamReader isreader = new InputStreamReader(bais, aEncoding);
			Source xmlSrc = new StreamSource(isreader);

			// get XSL Source file
			
			File fXSLTSourceFile = new File(xsltSourceFile);
			Source xsltSrc = new StreamSource(fXSLTSourceFile);

			Transformer trans = transFact.newTransformer(xsltSrc);
			OutputStream outStream = uriHandler.getResponse().getOutputStream();
			trans.transform(xmlSrc, new StreamResult(outStream));

			outStream.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method transforms an xml string with a provided xsl String
	 * 
	 * The XMLSource will be read from a InputStreamReader object to provide the
	 * XSLTransformer with a inputsream in expected encoding.
	 * 
	 * @param xmlSource
	 * @param xsltSourceFile
	 * @return
	 */
	protected void xslTranformation(String xmlSource, String xslSource,
			String aEncoding) {
		try {
			TransformerFactory transFact = TransformerFactory.newInstance();

			// generate XML InputStream Reader with encoding
			ByteArrayInputStream baisXML = new ByteArrayInputStream(xmlSource
					.getBytes());
			InputStreamReader isreaderXML = new InputStreamReader(baisXML,
					aEncoding);
			Source xmlSrc = new StreamSource(isreaderXML);

			// generate XSL InputStream Reader with encoding
			ByteArrayInputStream baisXSL = new ByteArrayInputStream(xslSource
					.getBytes());
			InputStreamReader isreaderXSL = new InputStreamReader(baisXSL,
					aEncoding);
			Source xslSrc = new StreamSource(isreaderXSL);

			Transformer trans = transFact.newTransformer(xslSrc);
			OutputStream outStream = uriHandler.getResponse().getOutputStream();
			trans.transform(xmlSrc, new StreamResult(outStream));

			outStream.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}



}
