package org.imixs.workflow.jee.rest.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.FactoryConfigurationError;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.EntityService;
import org.imixs.workflow.jee.ejb.ModelService;
import org.imixs.workflow.xml.EntityCollection;
import org.imixs.workflow.xml.XMLItemCollection;
import org.imixs.workflow.xml.XMLItemCollectionAdapter;

public class ModelServiceHandler extends AbstractRestServiceHandler {

	public ModelServiceHandler(URIHandler aurihandler) {
		super(aurihandler);

	}

	/**
	 * This method parses the xml request InputStream and translates the stream
	 * into a EntityCollection. Next the method extracts the Entity Objects and
	 * saves the transformed ItemCollection Objects using an instance of the
	 * EntityService EJB.
	 * 
	 * The Method test if a subresource with a ModelVersion is provided. If yes
	 * than the method will update each Entity object with the property
	 * $ModelVersion. In cases where no subversource is proviced the method
	 * takes the entities as they are. Only the first entity will be tested for
	 * a provided $modelVersion - so an old version will be automatically
	 * removed before update.
	 * 
	 * 
	 * @param entityService
	 * @throws FactoryConfigurationError
	 * @throws Exception
	 */
	public void updateModel(EntityService entityService,
			ModelService modelService) throws FactoryConfigurationError,
			Exception {

		ItemCollection itemCollection;
		XMLItemCollection entity;
		// OMElement responseElement = null;
		String sModelVersion = null;

		// extract model item collection from request stream.....

		InputStream is = this.uriHandler.getRequest().getInputStream();

		/*
		System.out.print(convertStreamToString(is));
		
if (1==1)
	return;
*/
		JAXBContext context = JAXBContext.newInstance(EntityCollection.class);

		Unmarshaller u = context.createUnmarshaller();
		EntityCollection ecol = (EntityCollection) u.unmarshal(is);

		if (ecol.getEntity().length > 0) {
			/*
			 * first we need to delete the old model if available. First test if
			 * a subresource with amodelversion is provided. if not we test the
			 * first entity for a model version
			 */
			sModelVersion = this.uriHandler.getSubresource();
			if (sModelVersion == null || "".equals(sModelVersion)) {
				// try to get model version from first entity
				entity = ecol.getEntity()[0];
				itemCollection = XMLItemCollectionAdapter.getItemCollection(entity);
				sModelVersion = itemCollection
						.getItemValueString("$ModelVersion");
			}
			if (sModelVersion == null)
				sModelVersion = "";

			// delete old model if a modelversion is available
			if (!"".equals(sModelVersion))
				modelService.removeModelVersion(sModelVersion);

			// save new entities into database and update modelversion.....
			for (int i = 0; i < ecol.getEntity().length; i++) {
				entity = ecol.getEntity()[i];
				itemCollection = XMLItemCollectionAdapter.getItemCollection(entity);
				// update model version
				itemCollection.replaceItemValue("$modelVersion", sModelVersion);
				// save entity
				entityService.save(itemCollection);
			}
		}

	}

	
	   public String convertStreamToString(InputStream is) throws IOException {
		       /*
		        * To convert the InputStream to String we use the BufferedReader.readLine()
		         * method. We iterate until the BufferedReader return null which means
		         * there's no more data to read. Each line will appended to a StringBuilder
		         * and returned as String.
		        */
		        if (is != null) {
		            StringBuilder sb = new StringBuilder();
		            String line;
		
		            try {
		               BufferedReader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
		               while ((line = reader.readLine()) != null) {
		                   sb.append(line).append("\n");
		                }
	           } finally {
		                is.close();
		           }
		          return sb.toString();
	       } else {       
	           return "";
		       }
		    }
}
