package org.imixs.workflow.jee.rest.servlet;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Vector;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.EntityService;

public class ReportServiceHandler extends AbstractRestServiceHandler {





	public ReportServiceHandler(URIHandler aurihandler) {
		super(aurihandler);
		
	}



	/**
	 * This method processes a report request. The method loads the report
	 * entity item (type=ReportEntity) indicated by its name (txtname) The name
	 * is indicated by the method restRequestParser.getUniqueID()
	 * 
	 * Next the method starts a EQL query against the entityService. The EQL
	 * query can contain dynamich params (e.g. ...?1=xxx....) which will be
	 * replaced by corresponding query params (e.g. &1=yyy)
	 * 
	 * For html output the results will be printed into a html table! the
	 * txtattributes of the queryitem are indicating the columns
	 * 
	 * @param out
	 * @throws Exception
	 */
	public void processReportRequest(EntityService entityService) throws Exception {
		setContentType();

		Collection<ItemCollection> col = null;

		// find report Workitem
		String sReportName = uriHandler.getSubresource();

		// add .ixr extension if extension is missed
		if (!sReportName.endsWith(".ixr"))
			sReportName = sReportName + ".ixr";

		String sQuery = null;
		sQuery = "SELECT report from Entity AS report "
				+ " JOIN report.textItems AS t2 "
				+ " WHERE report.type = 'ReportEntity'"
				+ " AND t2.itemName = 'txtname' " + " AND t2.itemValue = '"
				+ sReportName + "'";

		col = entityService.findAllEntities(sQuery, 0, 1);

		if (col.size() == 0)
			return;

		ItemCollection queryCol = col.iterator().next();
		// ItemCollection queryCol =
		// entityService.load(restRequestParser.getUniqueID());

		String sEQL = queryCol.getItemValueString("txtquery");

		// Now replace dynamic EQL params like
		// AND myindex.itemValue=?1
		sEQL = parseQueryForParams(sEQL);

		Vector<String> vAttributList = queryCol
				.getItemValue("txtAttributeList");
		col = entityService.findAllEntities(sEQL, uriHandler
				.getStartpos(), uriHandler.getCount());

		if (col != null) {
			if ("html".equals(uriHandler.getFormat()))
				// print html into a html table
				printWorkItemCollectionHTML(col, vAttributList);
			else if ("xml".equals(uriHandler.getFormat()))
				// generate a standard output in xml
				printWorkItemCollectionXML(col);
			else if ("txt".equals(uriHandler.getFormat()))
				// generate a standard output in xml
				printWorkItemCollectionText(col);
			else {
				// The default output is either XML or a XSL Transformation if
				// a xsl definition is provided by the report.

				// test if XSL transformation should be performed
				String sXSL = queryCol.getItemValueString("txtXSL");

				if (!"".equals(sXSL)) {
					// Transform XML per XSL and generate output
					String sContentType = queryCol
							.getItemValueString("txtcontenttype");
					String sEncoding = null;

					if ("".equals(sContentType))
						sContentType = "text/xml";

					// test if default encoding should be taken from URL request
					// or encoding is defined in report
					if (uriHandler.hasEncoding()) {
						sEncoding = uriHandler.getEncoding();
					} else {
						// no encoding was defined in request url. so take
						// encoding from report
						sEncoding = queryCol.getItemValueString("txtencoding");
						// no encoding defined so take a default encoding
						// (UTF-8)
						if ("".equals(sEncoding))
							sEncoding = uriHandler.getEncoding();
					}

					uriHandler.getResponse().setContentType(sContentType);

					uriHandler.getResponse().setHeader("Content-Type", sContentType
							+ ";charset=" + sEncoding);
					// settings to enable MS IE
					uriHandler.getResponse().setHeader("Pragma", "public");
					uriHandler.getResponse().setHeader("Cache-Control", "must-revalidate");

					// set Attachment filename header for direct download?
					String sDownloadName = uriHandler.getDownload();
					if (sDownloadName != null && !"".equals(sDownloadName))
						uriHandler.getResponse().setHeader("Content-disposition",
								"attachment;filename=" + sDownloadName);

					uriHandler.getResponse().setCharacterEncoding(sEncoding);

					String sXML = getWorkItemCollectionXML(col);
 
					xslTranformation(sXML, sXSL, sEncoding);

				} else
					// default XML output because no XSL was defined for
					// transformation
					printWorkItemCollectionXML(col);

			}
		} else {
			PrintWriter out = uriHandler.getResponse().getWriter();
			out.println("No workitems found");
			out.close();
		}

	}
	
	
	
	/**
	 * This method verifies if a EQL Statement from a report contains dynmaic
	 * paramas ('?n'). if yes the method checks to find out the corresponding
	 * value from the options list provided by the request.
	 * 
	 * @param aQuery
	 * @return
	 */
	private String parseQueryForParams(String aQuery) {
		int iSearchPos = 0;
		while (aQuery.indexOf("?", iSearchPos) > -1) {
			int iParamPos = aQuery.indexOf("?");
			iSearchPos = iParamPos + 1;
			// find out number of param
			String sNumber = "";
			int iPos = iParamPos + 1;
			while (true) {
				try {
					char sChar = aQuery.charAt(iPos);
					// test if char is a number?
					int i = Integer.parseInt("" + sChar);
					sNumber = sNumber + sChar;
					iPos++;
				} catch (NumberFormatException nfe) {
					break;
				} catch (IndexOutOfBoundsException eofbe) {
					break;
				}
			}
			// now replace param with corresponding options param
			int iValuePos = uriHandler.getOptions().indexOf(
					sNumber + "=");
			if (iValuePos > -1) {
				// a corresponding param is provided!
				String sParamValue = uriHandler.getOptions().substring(
						iValuePos + (sNumber + "=").length());
				// cut next param
				if (sParamValue.indexOf("&") > -1)
					sParamValue = sParamValue.substring(0, sParamValue
							.indexOf("&"));

				// now replace EQL Statement with the provided value from the
				// options list
				aQuery = aQuery.replace("?" + sNumber, sParamValue);
			}

		}

		return aQuery;

	}


}
