package org.imixs.workflow.jee.rest.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.imixs.workflow.ItemCollection;

/**
 * Servlet implementation class for Servlet: WorkflowRestService. The servlet
 * supports three different main URIs
 * <ul>
 * <li>/workflow
 * <li>/model
 * <li>/report
 * 
 * each main resource supports an individual set of URIs for the specific
 * module.
 * 
 * @version 0.0.2
 * @author rsoika
 * 
 */
public class WorkflowRestService extends javax.servlet.http.HttpServlet
		implements javax.servlet.Servlet {
	static final long serialVersionUID = 1L;
	HttpServletRequest request;
	HttpServletResponse response;
	URIHandler uriHandler = null;

	@EJB
	org.imixs.workflow.jee.ejb.WorkflowService workflowService;

	@EJB
	org.imixs.workflow.jee.ejb.ModelService modelService;

	@EJB
	org.imixs.workflow.jee.ejb.EntityService entityService;

	/*
	 * (non-Java-doc)
	 * 
	 * @see javax.servlet.http.HttpServlet#HttpServlet()
	 */
	public WorkflowRestService() {
		super();
	}

	/**
	 * This method receifes the get reqeuests. The method parses the request
	 * path to decide if a worklist or a single workitem should be displayed.
	 */
	protected void doGet(HttpServletRequest arequest,
			HttpServletResponse aresponse) throws ServletException, IOException {
		this.request = arequest;
		this.response = aresponse;
		try {
			// Check Request type
			uriHandler = new URIHandler(request, response, this
					.getServletContext());

			/*
			 * Process Workflow Requests
			 */
			if (uriHandler.isWorkflowRequest()
					&& (!"".equals(uriHandler.getSubresource()))) {

				WorkflowServiceHandler workflowHandler = new WorkflowServiceHandler(
						uriHandler);

				// output a File? other outputs are normal printer outputs....
				if (uriHandler.isFileRequest()) {
					workflowHandler.processFileRequest(workflowService);
					return;
				}

				// output a view?
				if (uriHandler.isCollectionRequest()) {
					workflowHandler.processViewRequest(workflowService);
					// out.close();
					return;
				}

				// default output a single WorkItem?
				workflowHandler.processWorkItemRequest(workflowService);
				return;

			}

			/*
			 * Process Report Requests
			 */
			if (uriHandler.isReportRequest()) {
				/*
				ReportServiceHandler reportHandler = new ReportServiceHandler(
						uriHandler);

				reportHandler.processReportRequest(entityService);
				*/
				return;
			}

			/*
			 * Process Model Requests
			 */
			if (uriHandler.isModelRequest()) {
				PrintWriter out = response.getWriter();
				response.setContentType("text/html");

				out.println("<html>");
				out.println("<body>");
				out.println("Model Resources not implemented");
				out.println("</body>");
				out.println("</html>");
				out.close();

				return;
			}

			// invalid request
			// PrintWriter out = null;
			PrintWriter out = response.getWriter();
			response.setContentType("text/html");

			out.println("<html>");
			AbstractRestServiceHandler.printHead(out);
			out.println("<body>");
			printHelp(out);

			out.println("</body>");
			out.println("</html>");
			out.close();

		} catch (Exception e) {
			PrintWriter out = response.getWriter();
			out.println("ERROR:");
			e.printStackTrace(out);
			out.close();
		}

	}

	/**
	 * Deletes a single workitem, report entity or model entity. The Delete
	 * method can be called for any of the main resources (WorkflowRequest,
	 * ReportRequest, ModelRequest) In all cases the method expects the uniqueid
	 * of the entity to be deleted. The caller needs to have at least author
	 * access to the specific entity.
	 * 
	 */
	@Override
	protected void doDelete(HttpServletRequest arequest,
			HttpServletResponse aresponse) throws ServletException, IOException {
		this.request = arequest;
		this.response = aresponse;

		// Check Request type
		uriHandler = new URIHandler(request, response, this.getServletContext());

		if (uriHandler.isWorkflowRequest() || uriHandler.isModelRequest()
				|| uriHandler.isReportRequest()) {

			PrintWriter out = response.getWriter();

			try {
				// get the uniqueid of the entity to be deleted
				String sUnqiueID = uriHandler.getSubresource();
				if (sUnqiueID == null) {
					// entity not found
					response.setStatus(HttpServletResponse.SC_NOT_FOUND);
				} else {
					ItemCollection itemCol = entityService.load(sUnqiueID);
					if (itemCol == null) {
						// entity not found
						response.setStatus(HttpServletResponse.SC_NOT_FOUND);
					} else {
						// try to delete worktiem
						entityService.remove(itemCol);
					}
				}

			} catch (Exception e) {
				out.println("ERROR<br>");
				e.printStackTrace(out);
			}
			out.close();
		}
	}

	/**
	 * Creates a new workitem
	 */
	@Override
	protected void doPut(HttpServletRequest arequest,
			HttpServletResponse aresponse) throws ServletException, IOException {

		this.request = arequest;
		this.response = aresponse;

		uriHandler = new URIHandler(request, response, this.getServletContext());

		if (uriHandler.isWorkflowRequest()) {

			WorkflowServiceHandler workflowHandler = new WorkflowServiceHandler(
					uriHandler);

			workflowHandler.setContentType();

			try {
				// get the workitem
				ItemCollection workItem = workflowService
						.getWorkItem(uriHandler.getSubresource());
				if (workItem != null) {
					// update params
					workItem = workflowHandler.updateWorkItem(workItem);
					workflowService.processWorkItem(workItem);

					if ("html".equals(uriHandler.getFormat())) {
						PrintWriter out = response.getWriter();

						workflowHandler.printWorkItemHTML(out, workItem);
						out.close();

					} else if ("xml".equals(uriHandler.getFormat()))
						workflowHandler.printWorkItemXML(workItem);
					else
						workflowHandler.printWorkItemText(workItem);
				} else {
					// workitem not found
					response.setStatus(HttpServletResponse.SC_NOT_FOUND);
				}

			} catch (Exception e) {
				response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
				PrintWriter out = response.getWriter();
				out.println("ERROR<br>");
				out.close();
				e.printStackTrace(out);
			}

		}

		if (uriHandler.isModelRequest()) {
			ModelServiceHandler modelHandler = new ModelServiceHandler(
					uriHandler);
			try {
				modelHandler.setContentType();
				modelHandler.updateModel(entityService, modelService);
			} catch (Exception e) {
				response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
				PrintWriter out = response.getWriter();
				out.println("ERROR<br>");
				out.close();
				e.printStackTrace(out);

			}
		}

	}

	/**
	 * this method creates or processes a new workitem parameter $processid is
	 * needed here. The methode calls the mehtod doPut if a valid $UniqueID is
	 * supported. This is because a PUT method is not allowed by browsers in a
	 * Form Tag. So the Post Method can handle both, the creation and the update
	 * of a workitem
	 * 
	 */
	protected void doPost(HttpServletRequest arequest,
			HttpServletResponse aresponse) throws ServletException, IOException {

		this.request = arequest;
		this.response = aresponse;

		uriHandler = new URIHandler(request, response, this.getServletContext());

		// for both - the WorkflowRequest and the ModelRequest a Post can be processed by doPut...
		if (uriHandler.isWorkflowRequest() || uriHandler.isModelRequest()) {
			this.doPut(arequest, aresponse);
		} else {
			response.setStatus(HttpServletResponse.SC_NOT_ACCEPTABLE);
			PrintWriter out = response.getWriter();
			out.println("ERROR<br>");
			out.close();
		}
	}

	/**
	 * prints a short help which is displayed after an invalid or an unclear
	 * request.
	 * 
	 * @param out
	 */
	private void printHelp(PrintWriter out) {

		out.println("<h1>Imixs Workflow REST Service</h1>");
		try {

		} catch (Exception e) {
			out.println("ERROR<br>");
			e.printStackTrace(out);
		}
		out.println("<h2>Resources :</h2>");

		out.println("<table><tbody>");

		out.println("<tr class=\"a\">");

		out.println("<th>Method</th><th>URI</th><th>Description</th></tr>");

		out.println("<tr class=\"b\">");

		out
				.println("<td>GET</td><td>/workflow/worklist</td><td>get all workitems from users worklist</td></tr>");

		out.println("<tr class=\"a\">");

		out
				.println("<td>GET</td><td>/workflow/$uniqueid</td><td>get one single workitem</td></tr>");

		out.println("<tr class=\"b\">");

		out
				.println("<td>GET</td><td>/report/$file</td><td>request a report by report .ixr file</td></tr>");

		out.println("</tbody></table>");

		out.println("<h2>Formats:</h2>");

		out.println("<table><tbody>");

		out.println("<tr class=\"a\">");

		out.println("<th>Method</th><th>URI</th><th>Description</th></tr>");

		out.println("<tr class=\"b\">");

		out
				.println("<td>GET</td><td>.xml</td><td>XML format (default)</td></tr>");

		out.println("<tr class=\"a\">");

		out.println("<td>GET</td><td>.txt</td><td>Text format</td></tr>");

		out.println("<tr class=\"b\">");

		out.println("<td>GET</td><td>.html</td><td>HTML format</td></tr>");

		out.println("</tbody></table>");

		out
				.println("<br/><br/><br/><br/>Version 2.0.2 - See <a href=\"http://www.imixs.com\">Imixs Workflow Project Home</a> for further informations.");

	}

}