package org.imixs.workflow.jee.rest.servlet;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.servlet.http.HttpServletResponse;

import org.imixs.workflow.ItemCollection;
import org.imixs.workflow.jee.ejb.WorkflowService;

/**
 * The WorkflowService Handler supports methods to process different kind of
 * request URIs
 * 
 * @author rsoika
 * 
 */
public class WorkflowServiceHandler extends AbstractRestServiceHandler {

	public WorkflowServiceHandler(URIHandler aurihandler) {
		super(aurihandler);
		
	}


	/**
	 * This method processes a worklist request. The method tries to indicate
	 * the right workflowmanager method by parsing the view name
	 * 
	 * @param out
	 * @throws Exception
	 */
	public void processViewRequest(WorkflowService workflowService)
			throws Exception {
		setContentType();

		Collection<ItemCollection> col = null;
		if ("worklist".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkList(null, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);
		if ("workitems".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkListByCreator(null, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);
		if ("worklistbycreator".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkListByCreator(null, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);
		if ("worklistbygroup".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkListByGroup(null, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);

		if ("worklistbyprocessid".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkListByProcessID(0, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);
		if ("worklistbyref".equals(uriHandler.getSubresource()))
			col = workflowService.getWorkListByRef(null, uriHandler
					.getStartpos(), uriHandler.getCount(),null,0);

		if (col != null) {
			if ("html".equals(uriHandler.getFormat())) {

				printWorkItemCollectionHTML( col);

			} else if ("xml".equals(uriHandler.getFormat()))
				printWorkItemCollectionXML( col);
			else
				printWorkItemCollectionText( col);

		} else {
			PrintWriter out = uriHandler.getResponse().getWriter();
			out.println("View is not defined");
			out.close();

		}

	}
	
	
	/**
	 * This method posts a workItemReqeust The method print the output depending
	 * on the format in the query string ?open = text ?openHTML = html ?openXML
	 * = xml ?openJSON = json
	 * 
	 * @param out
	 * @param aWorkItemID
	 * @throws Exception
	 */
	public void processWorkItemRequest(WorkflowService workflowService) throws Exception {
		setContentType();

		String aWorkItemID=uriHandler.getSubresource();
		ItemCollection workItem = workflowService.getWorkItem(aWorkItemID);
		if (workItem != null) {
			if ("html".equals(uriHandler.getFormat())) {
				PrintWriter out = uriHandler.getResponse().getWriter();

				out.println("<html>");
				printHead(out); 
				out.println("<body>"); 

				printWorkItemHTML(out, workItem);

				out.println("</body>");
				out.println("</html>");
				out.close();

			} else if ("xml".equals(uriHandler.getFormat()))
				printWorkItemXML(workItem);
			else
				printWorkItemText(workItem);

		} else {
			// workitem not found
			uriHandler.getResponse().setStatus(HttpServletResponse.SC_NOT_FOUND);
		}

	}
	
	

	/**
	 * This method output a requested File inside a workitem. Files should be
	 * stored in the attribute $file.
	 * 
	 * @param out
	 * @throws Exception
	 */
	public void processFileRequest(WorkflowService workflowService )
			throws Exception {

		
		String aWorkItemID=uriHandler.getSubresource();
		String aFileName=uriHandler.getFile();
		
		ItemCollection workItem = workflowService.getWorkItem(aWorkItemID);
		if (workItem != null) {
			// fetch $file
			// Store files using a hashmap....
			HashMap mapFiles = null;
			Vector vFiles = workItem.getItemValue("$file");
			if (vFiles != null && vFiles.size() > 0) {
				mapFiles = (HashMap) vFiles.elementAt(0);

				Vector<Object> vectorFileInfo = new Vector<Object>();
				vectorFileInfo = (Vector) mapFiles.get(aFileName);
				if (vectorFileInfo != null) {
					String sContentType = vectorFileInfo.elementAt(0)
							.toString();
					byte[] fileContent = (byte[]) vectorFileInfo.elementAt(1);

					// Set content type in order of the contentType stored
					// in the $file attribute
					uriHandler.getResponse().setContentType(sContentType);
					uriHandler.getResponse().setContentLength((int) fileContent.length);
					uriHandler.getResponse().setCharacterEncoding("UTF-8");

					OutputStream outStream = uriHandler.getResponse().getOutputStream();

					outStream.write(fileContent);
					outStream.close();

				} else
					// workitem not found
					uriHandler.getResponse().setStatus(HttpServletResponse.SC_NOT_FOUND);

			} else
				// workitem not found
				uriHandler.getResponse().setStatus(HttpServletResponse.SC_NOT_FOUND);

		} else {
			// workitem not found
			uriHandler.getResponse().setStatus(HttpServletResponse.SC_NOT_FOUND);
		}

	}

	
	/**
	 * This method updates a workitem with the parameters supported by the
	 * request object
	 * 
	 * @param aworkitem
	 * @return
	 */
	public ItemCollection updateWorkItem(ItemCollection aworkitem) {

		Enumeration params = uriHandler.getRequest().getParameterNames();
		while (params.hasMoreElements()) {
			String sName = params.nextElement().toString();

			try {
				if (isValidParam(sName)) {
					String sValue = uriHandler.getRequest().getParameter(sName);
					// check type
					Vector oldvalues = aworkitem.getItemValue(sName);
					if (oldvalues.size() > 0) {
						Object o = oldvalues.firstElement();
						if (o instanceof Integer)
							aworkitem.replaceItemValue(sName, new Integer(
									sValue));
						else if (o instanceof Double)
							aworkitem.replaceItemValue(sName,
									new Double(sValue));
						else
							aworkitem.replaceItemValue(sName, sValue);
					} else {
						// new attribute
						if (!"".equals(sValue))
							aworkitem.replaceItemValue(sName, sValue);
					}
				}

			} catch (Exception e) {
				// no action
			}
		}

		return aworkitem;
	}
	

	/**
	 * This method checks if a supported param name is accaptable
	 * 
	 * @param aName
	 * @return
	 */
	private boolean isValidParam(String aName) {
		aName = aName.toLowerCase();
		if ("$uniqueid".equals(aName))
			return false;

		return true;
	}

}
